import base64
from collections.abc import Callable
import os
import time
from typing import Any

from google import genai
from google.genai import types
from google.genai.types import FunctionCall, FunctionCallingConfigMode, GenerateContentResponse

from not_again_ai.llm.chat_completion.types import (
    AssistantMessage,
    ChatCompletionChoice,
    ChatCompletionRequest,
    ChatCompletionResponse,
    Function,
    ImageContent,
    Role,
    TextContent,
    ToolCall,
)

# This should be all of the options we want to support in types.GenerateContentConfig, that are not handled otherwise
GEMINI_PARAMETER_MAP = {
    "max_completion_tokens": "max_output_tokens",
    "temperature": "temperature",
    "top_p": "top_p",
    "top_k": "top_k",
}

GEMINI_FINISH_REASON_MAP = {
    "STOP": "stop",
    "MAX_TOKENS": "max_tokens",
    "SAFETY": "safety",
    "RECITATION": "recitation",
    "LANGUAGE": "language",
    "OTHER": "other",
    "BLOCKLIST": "blocklist",
    "PROHIBITED_CONTENT": "prohibited_content",
    "SPII": "spii",
    "MALFORMED_FUNCTION_CALL": "malformed_function_call",
    "IMAGE_SAFETY": "image_safety",
}


def gemini_chat_completion(request: ChatCompletionRequest, client: Callable[..., Any]) -> ChatCompletionResponse:
    """Experimental Gemini chat completion function."""
    # Handle messages
    # Any system messages need to be removed from messages and concatenated into a single string (in order)
    system = ""
    contents = []
    for message in request.messages:
        if message.role == "system":
            # Handle both string content and structured content
            if isinstance(message.content, str):
                system += message.content + "\n"
            else:
                # If it's a list of content parts, extract text content
                for part in message.content:
                    if hasattr(part, "text"):
                        system += part.text + "\n"
        elif message.role == "tool":
            tool_name = message.name if message.name is not None else ""
            function_response_part = types.Part.from_function_response(
                name=tool_name,
                response={"result": message.content},
            )
            contents.append(
                types.Content(
                    role="user",
                    parts=[function_response_part],
                )
            )
        elif message.role == "assistant":
            if message.content and isinstance(message.content, str):
                contents.append(types.Content(role="model", parts=[types.Part(text=message.content)]))
            function_parts = []
            if isinstance(message, AssistantMessage) and message.tool_calls:
                for tool_call in message.tool_calls:
                    function_call_part = types.Part(
                        function_call=FunctionCall(
                            id=tool_call.id,
                            name=tool_call.function.name,
                            args=tool_call.function.arguments,
                        )
                    )
                    function_parts.append(function_call_part)
            if function_parts:
                contents.append(types.Content(role="model", parts=function_parts))
        elif message.role == "user":
            if isinstance(message.content, str):
                contents.append(types.Content(role="user", parts=[types.Part(text=message.content)]))
            elif isinstance(message.content, list):
                parts = []
                for part in message.content:
                    if isinstance(part, TextContent):
                        parts.append(types.Part(text=part.text))
                    elif isinstance(part, ImageContent):
                        # Extract MIME type and data from data URI
                        uri_parts = part.image_url.url.split(",", 1)
                        if len(uri_parts) == 2:
                            mime_type = uri_parts[0].split(":")[1].split(";")[0]
                            base64_data = uri_parts[1]
                            image_data = base64.b64decode(base64_data)
                            parts.append(types.Part.from_bytes(mime_type=mime_type, data=image_data))
                contents.append(types.Content(role="user", parts=parts))

    kwargs: dict[str, Any] = {}
    kwargs["contents"] = contents
    kwargs["model"] = request.model
    config: dict[str, Any] = {}
    config["system_instruction"] = system.rstrip()
    config["automatic_function_calling"] = {"disable": True}

    # Handle the possible tool choice options
    if request.tool_choice:
        tool_choice = request.tool_choice
        if tool_choice == "auto":
            config["tool_config"] = types.FunctionCallingConfig(mode=FunctionCallingConfigMode.AUTO)
        elif tool_choice == "any":
            config["tool_config"] = types.FunctionCallingConfig(mode=FunctionCallingConfigMode.ANY)
        elif tool_choice == "none":
            config["tool_config"] = types.FunctionCallingConfig(mode=FunctionCallingConfigMode.NONE)
        elif isinstance(tool_choice, list):
            config["tool_config"] = types.FunctionCallingConfig(
                mode=FunctionCallingConfigMode.ANY, allowed_function_names=tool_choice
            )
        elif tool_choice not in (None, "auto", "any", "none"):
            config["tool_config"] = types.FunctionCallingConfig(
                mode=FunctionCallingConfigMode.ANY, allowed_function_names=[tool_choice]
            )

    # Handle tools
    tools = []
    for tool in request.tools or []:
        tools.append(types.Tool(function_declarations=[tool]))  # type: ignore
    if tools:
        config["tools"] = tools

    # Everything else defined in GEMINI_PARAMETER_MAP goes into kwargs["config"]
    request_kwargs = request.model_dump(mode="json", exclude_none=True)
    for key, value in GEMINI_PARAMETER_MAP.items():
        if value is not None and key in request_kwargs:
            config[value] = request_kwargs.pop(key)

    kwargs["config"] = types.GenerateContentConfig(**config)

    start_time = time.time()
    response: GenerateContentResponse = client(**kwargs)
    end_time = time.time()
    response_duration = round(end_time - start_time, 4)

    finish_reason = "other"
    if response.candidates and response.candidates[0].finish_reason:
        finish_reason_str = str(response.candidates[0].finish_reason)
        finish_reason = GEMINI_FINISH_REASON_MAP.get(finish_reason_str, "other")

    tool_calls: list[ToolCall] = []
    tool_call_objs = response.function_calls
    if tool_call_objs:
        for tool_call_obj in tool_call_objs:
            tool_call_id = tool_call_obj.id if tool_call_obj.id else ""
            tool_calls.append(
                ToolCall(
                    id=tool_call_id,
                    function=Function(
                        name=tool_call_obj.name if tool_call_obj.name is not None else "",
                        arguments=tool_call_obj.args if tool_call_obj.args is not None else {},
                    ),
                )
            )

    assistant_message = ""
    if (
        response.candidates
        and response.candidates[0].content
        and response.candidates[0].content.parts
        and response.candidates[0].content.parts[0].text
    ):
        assistant_message = response.candidates[0].content.parts[0].text

    choice = ChatCompletionChoice(
        message=AssistantMessage(
            role=Role.ASSISTANT,
            content=assistant_message,
            tool_calls=tool_calls,
        ),
        finish_reason=finish_reason,
    )

    completion_tokens = 0
    # Add null check for usage_metadata
    if response.usage_metadata is not None:
        if response.usage_metadata.thoughts_token_count:
            completion_tokens = response.usage_metadata.thoughts_token_count
        if response.usage_metadata.candidates_token_count:
            completion_tokens += response.usage_metadata.candidates_token_count

    # Set safe default for prompt_tokens
    prompt_tokens = 0
    if response.usage_metadata is not None and response.usage_metadata.prompt_token_count:
        prompt_tokens = response.usage_metadata.prompt_token_count

    chat_completion_response = ChatCompletionResponse(
        choices=[choice],
        completion_tokens=completion_tokens,
        prompt_tokens=prompt_tokens,
        response_duration=response_duration,
    )
    return chat_completion_response


def create_client_callable(client_class: type[genai.Client], **client_args: Any) -> Callable[..., Any]:
    """Creates a callable that instantiates and uses a Google genai client.

    Args:
        client_class: The Google genai client class to instantiate
        **client_args: Arguments to pass to the client constructor

    Returns:
        A callable that creates a client and returns completion results
    """
    filtered_args = {k: v for k, v in client_args.items() if v is not None}

    def client_callable(**kwargs: Any) -> Any:
        client = client_class(**filtered_args)
        completion = client.models.generate_content(**kwargs)
        return completion

    return client_callable


def gemini_client(api_key: str | None = None) -> Callable[..., Any]:
    if not api_key:
        api_key = os.environ.get("GEMINI_API_KEY")
    client_callable = create_client_callable(genai.Client, api_key=api_key)
    return client_callable



from pathlib import Path
from typing import Any

from pydantic import BaseModel, Field


class ImageGenRequest(BaseModel):
    prompt: str
    model: str
    images: list[Path] | None = Field(default=None)
    mask: Path | None = Field(default=None)
    n: int = Field(default=1)
    quality: str | None = Field(default=None)
    size: str | None = Field(default=None)
    background: str | None = Field(default=None)
    moderation: str | None = Field(default=None)


class ImageGenResponse(BaseModel):
    images: list[bytes]
    input_tokens: int
    output_tokens: int
    response_duration: float
    input_tokens_details: dict[str, Any] | None = Field(default=None)



import asyncio
import io
import mimetypes
from pathlib import Path
import re
from urllib.parse import urlparse

from crawl4ai import AsyncWebCrawler, CacheMode
from crawl4ai.async_configs import BrowserConfig, CrawlerRunConfig
from crawl4ai.markdown_generation_strategy import DefaultMarkdownGenerator
import httpx
from markitdown import MarkItDown, StreamInfo
from pydantic import BaseModel


class Link(BaseModel):
    url: str
    text: str


class URLResult(BaseModel):
    url: str
    markdown: str
    links: list[Link] = []


async def _markitdown_bytes_to_str(file_bytes: bytes, filename_extension: str) -> str:
    """
    Convert a file using MarkItDown defaults.
    """
    with io.BytesIO(file_bytes) as temp:
        result = await asyncio.to_thread(
            MarkItDown(enable_plugins=False).convert,
            source=temp,
            stream_info=StreamInfo(extension=filename_extension),
        )
        text = result.text_content
    return text


def _detect_pdf_extension(url: str) -> bool:
    """
    Detect if the URL is a PDF based on its extension.
    """
    parsed_url = urlparse(url)
    filename = Path(parsed_url.path).name
    return mimetypes.guess_type(filename)[0] == "application/pdf"


def _detect_google_sheets(url: str) -> bool:
    """
    Detect if the URL is a Google Sheets document.
    """
    is_google_sheets = url.startswith("https://docs.google.com/spreadsheets/")
    return is_google_sheets


async def _handle_pdf_content(url: str) -> URLResult:
    md = MarkItDown(enable_plugins=False)
    result = md.convert(url)
    url_result = URLResult(
        url=url,
        markdown=result.markdown or "",
        links=[],
    )
    return url_result


async def _handle_google_sheets_content(url: str) -> URLResult:
    """
    Handle Google Sheets by using the export URL to get the raw content.
    """
    edit_pattern = r"https://docs\.google\.com/spreadsheets/d/([a-zA-Z0-9-_]+)/edit"
    export_pattern = r"https://docs\.google\.com/spreadsheets/d/([a-zA-Z0-9-_]+)/export\?format=csv"

    # Check if it's already an export URL
    export_match = re.search(export_pattern, url)
    if export_match:
        export_url = url
    else:
        # Check if it's an edit URL and extract document ID
        edit_match = re.search(edit_pattern, url)
        if edit_match:
            doc_id = edit_match.group(1)
            export_url = f"https://docs.google.com/spreadsheets/d/{doc_id}/export?format=csv&gid=0"
        else:
            return await _handle_web_content(url)

    async with httpx.AsyncClient(follow_redirects=True) as client:
        response = await client.get(export_url)
        response.raise_for_status()
        csv_bytes = response.content

    # Convert CSV to markdown using MarkItDown
    markdown_content = await _markitdown_bytes_to_str(csv_bytes, ".csv")

    url_result = URLResult(
        url=url,
        markdown=markdown_content,
        links=[],
    )
    return url_result


async def _handle_web_content(url: str) -> URLResult:
    browser_config = BrowserConfig(
        browser_type="chromium",
        headless=True,
        verbose=False,
        user_agent_mode="random",
        java_script_enabled=True,
    )
    run_config = CrawlerRunConfig(
        scan_full_page=True,
        user_agent_mode="random",
        cache_mode=CacheMode.DISABLED,
        markdown_generator=DefaultMarkdownGenerator(),
    )

    async with AsyncWebCrawler(config=browser_config) as crawler:
        result = await crawler.arun(
            url=url,
            config=run_config,
        )

        if result.response_headers.get("content-type") == "application/pdf":
            return await _handle_pdf_content(url)

    links: list[Link] = []
    seen_urls: set[str] = set()
    combined_link_data = result.links.get("internal", []) + result.links.get("external", [])
    for link_data in combined_link_data:
        href = link_data.get("href", "")
        if href and href not in seen_urls:
            seen_urls.add(href)
            link = Link(
                url=href,
                text=link_data.get("title", "") or link_data.get("text", ""),
            )
            links.append(link)

    url_result = URLResult(
        url=url,
        markdown=result.markdown or "",
        links=links,
    )
    return url_result


async def process_url(url: str) -> URLResult:
    """
    Process a URL to extract content and convert it to Markdown and links
    """
    if _detect_pdf_extension(url):
        url_result = await _handle_pdf_content(url)
    elif _detect_google_sheets(url):
        url_result = await _handle_google_sheets_content(url)
    else:
        url_result = await _handle_web_content(url)
    return url_result



from collections.abc import Callable
from typing import Any

from not_again_ai.llm.image_gen.providers.openai_api import openai_create_image
from not_again_ai.llm.image_gen.types import ImageGenRequest, ImageGenResponse


def create_image(request: ImageGenRequest, provider: str, client: Callable[..., Any]) -> ImageGenResponse:
    """Get a image response from the given provider. Currently supported providers:
    - `openai` - OpenAI
    - `azure_openai` - Azure OpenAI

    Args:
        request: Request parameter object
        provider: The supported provider name
        client: Client information, see the provider's implementation for what can be provided

    Returns:
        ImageGenResponse: The image generation response.
    """
    if provider == "openai" or provider == "azure_openai":
        return openai_create_image(request, client)
    else:
        raise ValueError(f"Provider {provider} not supported")



from typing import Any

import pytest

from not_again_ai.data.brave_search_api import search, search_news


@pytest.mark.parametrize(
    ("query", "search_params"),
    [
        ("brave search", {}),
        ("python programming", {"count": 2, "country": "US"}),
        ("machine learning", {"count": 4, "search_lang": "en", "freshness": "pw"}),
        ("AI news", {"count": 1, "offset": 5, "country": "GB", "ui_lang": "en-GB"}),
    ],
)
async def test_brave_search_api(query: str, search_params: dict[str, Any]) -> None:
    """Test the Brave Search API with a sample query and optional parameters."""
    content = await search(query=query, **search_params)
    assert content.results, f"No results returned for query: {query}"


@pytest.mark.skip("API Cost")
@pytest.mark.parametrize(
    ("query", "search_params"),
    [
        ("latest tech news", {}),
        ("AI breakthrough", {"count": 3, "country": "US"}),
    ],
)
async def test_brave_search_news_api(query: str, search_params: dict[str, Any]) -> None:
    """Test the Brave News Search API with a sample query and optional parameters."""
    content = await search_news(query=query, **search_params)
    assert content.results, f"No news results returned for query: {query}"



import os

import httpx
from loguru import logger
from pydantic import BaseModel


class SearchWebResult(BaseModel):
    title: str
    url: str
    description: str
    netloc: str | None = None


class SearchWebResults(BaseModel):
    results: list[SearchWebResult]


async def search(
    query: str,
    count: int = 20,
    offset: int = 0,
    country: str = "US",
    search_lang: str = "en",
    ui_lang: str = "en-US",
    freshness: str | None = None,
    timezone: str = "America/New_York",
    state: str = "MA",
    user_agent: str = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/134.0.0.0 Safari/537.36 Edg/134.0.0.",
) -> SearchWebResults:
    """
    Search using Brave Search API.

    Args:
        query: The search query string
        count: Number of search results to return (1-20, default 10)
        offset: Number of search results to skip (default 0)
        country: Country code for search results (default "US")
        search_lang: Language for search (default "en")
        ui_lang: User interface language (default "en-US")
        freshness: Freshness of results ("pd", "pw", "pm", "py" or YYYY-MM-DDtoYYYY-MM-DD or None)
        timezone: Timezone for search results (default "America/New_York")
        state: State for search results (default "MA")
        user_agent: User agent string for the request (default is a common browser UA)

    Returns:
        SearchWebResults: A model containing the search results

    Raises:
        httpx.HTTPError: If the request fails
        ValueError: If BRAVE_SEARCH_API_KEY is not set
    """
    api_key = os.getenv("BRAVE_SEARCH_API_KEY")
    if not api_key:
        raise ValueError("BRAVE_SEARCH_API_KEY environment variable is not set")

    url = "https://api.search.brave.com/res/v1/web/search"

    headers = {
        "Accept": "application/json",
        "Accept-Encoding": "gzip",
        "X-Subscription-Token": api_key,
        "X-Loc-Country": country,
        "X-Loc-Timezone": timezone,
        "X-Loc-State": state,
        "User-Agent": user_agent,
    }

    params: dict[str, str | int | bool] = {
        "q": query,
        "count": count,
        "offset": offset,
        "country": country,
        "search_lang": search_lang,
        "ui_lang": ui_lang,
        "text_decorations": False,
        "spellcheck": False,
        "units": "imperial",
        "extra_snippets": False,
        "safesearch": "off",
    }

    # Add optional parameters if provided
    if freshness:
        params["freshness"] = freshness

    try:
        async with httpx.AsyncClient() as client:
            response = await client.get(url, headers=headers, params=params)
            response.raise_for_status()
            data = response.json()
            results_list: list[SearchWebResult] = []
            for item in data.get("web", {}).get("results", []):
                result = SearchWebResult(
                    title=item.get("title", ""),
                    url=item.get("url", ""),
                    description=item.get("snippet", ""),
                    netloc=item.get("meta_url", {}).get("netloc", None),
                )
                results_list.append(result)
            return SearchWebResults(results=results_list)

    except httpx.HTTPError as e:
        logger.error(f"HTTP error during Brave search: {e}")
        raise
    except Exception as e:
        logger.error(f"Unexpected error during Brave search: {e}")
        raise


class SearchNewsResult(BaseModel):
    title: str
    url: str
    description: str
    age: str
    netloc: str | None = None


class SearchNewsResults(BaseModel):
    results: list[SearchNewsResult]


async def search_news(
    query: str,
    count: int = 20,
    offset: int = 0,
    country: str = "US",
    search_lang: str = "en",
    ui_lang: str = "en-US",
    freshness: str | None = None,
    user_agent: str = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/134.0.0.0 Safari/537.36 Edg/134.0.0.",
) -> SearchNewsResults:
    """
    Search news using Brave News Search API.

    Args:
        query: The search query string
        count: Number of news results to return (1-20, default 20)
        offset: Number of search results to skip (default 0)
        country: Country code for search results (default "US")
        search_lang: Language for search (default "en")
        ui_lang: User interface language (default "en-US")
        freshness: Freshness of results ("pd", "pw", "pm", "py" or YYYY-MM-DDtoYYYY-MM-DD or None)
        user_agent: User agent string for the request (default is a common browser UA)

    Returns:
        SearchNewsResults: A model containing the news search results

    Raises:
        httpx.HTTPError: If the request fails
        ValueError: If BRAVE_SEARCH_API_KEY is not set
    """
    api_key = os.getenv("BRAVE_SEARCH_API_KEY")
    if not api_key:
        raise ValueError("BRAVE_SEARCH_API_KEY environment variable is not set")

    url = "https://api.search.brave.com/res/v1/news/search"

    headers = {
        "Accept": "application/json",
        "Accept-Encoding": "gzip",
        "X-Subscription-Token": api_key,
        "User-Agent": user_agent,
    }

    params: dict[str, str | int | bool] = {
        "q": query,
        "count": count,
        "offset": offset,
        "country": country,
        "search_lang": search_lang,
        "ui_lang": ui_lang,
        "spellcheck": False,
        "safesearch": "off",
    }

    # Add optional parameters if provided
    if freshness:
        params["freshness"] = freshness

    try:
        async with httpx.AsyncClient() as client:
            response = await client.get(url, headers=headers, params=params)
            response.raise_for_status()
            data = response.json()
            results_list: list[SearchNewsResult] = []
            for item in data.get("results", []):
                result = SearchNewsResult(
                    title=item.get("title", ""),
                    url=item.get("url", ""),
                    description=item.get("description", ""),
                    age=item.get("age"),
                    netloc=item.get("meta_url", {}).get("netloc", None),
                )
                results_list.append(result)
            return SearchNewsResults(results=results_list)

    except httpx.HTTPError as e:
        logger.error(f"HTTP error during Brave news search: {e}")
        raise
    except Exception as e:
        logger.error(f"Unexpected error during Brave news search: {e}")
        raise



import pytest

from not_again_ai.data.web import process_url


@pytest.mark.parametrize(
    "url",
    [
        "https://example.com",
        "https://github.com/unclecode/crawl4ai",
        "https://arxiv.org/pdf/1710.02298",
        "https://www.youtube.com/watch?v=dQw4w9WgXcQ",
        "https://www.nascar.com/news/nascar-craftsman-truck-series/",
        "https://docs.google.com/spreadsheets/d/1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms/edit?gid=0#gid=0",
        "https://docs.google.com/spreadsheets/d/1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms/export?format=csv&gid=0",
    ],
)
async def test_process_url(url: str) -> None:
    content = await process_url(url)
    assert content, f"Content should not be empty for URL: {url}"



from collections.abc import Callable
from pathlib import Path
from typing import Any

import pytest

from not_again_ai.llm.image_gen import ImageGenRequest, create_image
from not_again_ai.llm.image_gen.providers.openai_api import openai_client

image_dir = Path(__file__).parents[1] / "sample_images"
body_lotion_image_path = image_dir / "body_lotion.png"
soap_image_path = image_dir / "soap.png"
sunlit_lounge_image_path = image_dir / "sunlit_lounge.png"
sunlit_lounge_mask_image_path = image_dir / "sunlit_lounge_mask.png"

save_dir = Path(__file__).parents[3] / ".nox" / "temp"
save_dir.mkdir(parents=True, exist_ok=True)
temp_image_path = save_dir / "temp_image.png"


@pytest.fixture(
    params=[
        {},
    ]
)
def openai_aoai_client_fixture(request: pytest.FixtureRequest) -> Callable[..., Any]:
    return openai_client(**request.param)


def test_create_image(openai_aoai_client_fixture: Callable[..., Any]) -> None:
    prompt = """A children's book drawing of a veterinarian using a stethoscope to 
listen to the heartbeat of a baby otter."""
    request = ImageGenRequest(
        prompt=prompt,
        model="gpt-image-1",
        quality="low",
    )
    response = create_image(request, "openai", openai_aoai_client_fixture)

    with temp_image_path.open("wb") as f:
        f.write(response.images[0])


def test_edit_image(openai_aoai_client_fixture: Callable[..., Any]) -> None:
    prompt = """Generate a photorealistic image of a gift basket on a white background 
labeled 'Relax & Unwind' with a ribbon and handwriting-like font, 
containing all the items in the reference pictures."""
    request = ImageGenRequest(
        prompt=prompt,
        model="gpt-image-1",
        images=[body_lotion_image_path],
        quality="low",
    )
    response = create_image(request, "openai", openai_aoai_client_fixture)

    with temp_image_path.open("wb") as f:
        f.write(response.images[0])


def test_edit_images(openai_aoai_client_fixture: Callable[..., Any]) -> None:
    prompt = """Generate a photorealistic image of a gift basket on a white background 
labeled 'Relax & Unwind' with a ribbon and handwriting-like font, 
containing all the items in the reference pictures."""
    request = ImageGenRequest(
        prompt=prompt,
        model="gpt-image-1",
        images=[body_lotion_image_path, soap_image_path],
        quality="low",
        size="1024x1024",
    )
    response = create_image(request, "openai", openai_aoai_client_fixture)

    with temp_image_path.open("wb") as f:
        f.write(response.images[0])


def test_edit_image_with_mask(openai_aoai_client_fixture: Callable[..., Any]) -> None:
    request = ImageGenRequest(
        prompt="A sunlit indoor lounge area with a pool containing a flamingo",
        model="gpt-image-1",
        images=[sunlit_lounge_image_path],
        mask=sunlit_lounge_mask_image_path,
        quality="low",
        size="1024x1024",
    )
    response = create_image(request, "openai", openai_aoai_client_fixture)
    with temp_image_path.open("wb") as f:
        f.write(response.images[0])


def test_create_image_multiple(openai_aoai_client_fixture: Callable[..., Any]) -> None:
    prompt = """A high-quality 3D-rendered illustration of a color wheel logo. \
The design features eight symmetrical, petal-shaped leaves arranged in a perfect circular flower pattern. \
Each leaf is semi-transparent like colored glass, rendered in soft pastel tones including pink, orange, yellow, green, blue, and purple. \
The petals overlap slightly, creating gentle blended hues where they intersect. \
The background is a flat, light-toned surface with even, diffused lighting, giving the image a modern, polished, and professional appearance. No text."""

    request = ImageGenRequest(
        prompt=prompt,
        model="gpt-image-1",
        n=2,
        quality="medium",
        size="1536x1024",
        background="transparent",
        moderation="low",
    )

    response = create_image(request, "openai", openai_aoai_client_fixture)

    with temp_image_path.open("wb") as f:
        f.write(response.images[0])
    with (temp_image_path.parent / "temp_image_1.png").open("wb") as f:
        f.write(response.images[1])
